require 'fileutils'

module Uricp::Strategy
  module CacheCommon
    def validate_cache!
      return if dry_run?

      unless cache_exists?
        raise Uricp::MissingCache,
              "no cache found at #{cache_root}. Expected a 'cache' and 'temp' directory"
      end
      return if cache_name

      raise Uricp::MissingCache,
            'no cache name found'
    end

    def in_cache?
      File.readable? cache_file
    end

    def cache_root
      options['cache']
    end

    def cache_name
      options['cache_name']
    end

    def cache_exists?
      cache_root && %w[temp cache].all? do |d|
        File.directory?(File.join(cache_root, d))
      end
    end

    def temp_cache_file
      @temp_cache_file ||= File.join(cache_root, 'temp', cache_name)
    end

    def temp_cache_uri
      URI.join('file:///', temp_cache_file)
    end

    def cache_file
      @cache_file ||= File.join(cache_root, 'cache', cache_name)
    end
  end
end
