module Uricp::Strategy
  class SegmentedRemotePut
    include Uricp::Strategy::Common

    def appropriate?
      unless segmented?
        debug "#{self.class.name}: not appropriate"
        return false
      end
      if compression_required? || conversion_required?
        debug "#{self.class.name}: not ready to upload"
        return false
      end
      case to.scheme
      when 'http', 'https'
        case from.scheme
        when 'file', 'pipe'
          return proposal if http_authentication?
        end
      end
      unsupported_transfer
      false
    end

    def command
      "segment_upload --segment-size '#{options['segment-size']}' #{source_details} #{to};"
    end

    def proposal
      @proposed_options = options.dup
      @proposed_options['from_uri'] = to
      @proposed_options['segment-size'] = nil
      self
    end

    def source_details
      "--from #{from.path}" unless from.scheme == 'pipe'
    end
  end
end
