require 'uri'

module Uricp
  class UriStrategy
    include Methadone::CLILogging

    def self.choose_strategy(options)
      current = options.reject { |k, _v| k.is_a? Symbol }
      strategy_list = []
      while STRATEGIES.detect { |klass| @strategy = klass.new(current).appropriate? }
        debug "#{name}: Selected strategy #{@strategy.class.name}"
        strategy_list << @strategy
        current = @strategy.proposed_options
      end
      if incomplete_strategy?(strategy_list)
        raise Uricp::UnsupportedURLtype, "Unsupported transfer from #{options['from_uri']} to #{options['to_uri']}"
      end

      strategy_list
    end

    def self.incomplete_strategy?(list)
      list.empty?
    end

    # This is an ordered list from the most specific to the most general
    STRATEGIES = [
      Strategy::CachedGet,
      Strategy::PipedRemoteGet,
      Strategy::PipedRbdGet,
      Strategy::PipedCacheConvert,
      Strategy::PipedCache,
      Strategy::PipedLocalDecompress,
      Strategy::PipedDecompress,
      Strategy::LocalConvert,
      Strategy::LocalLink,
      Strategy::PipedCompress,
      Strategy::RbdRemotePut,
      Strategy::SegmentedRemotePut,
      Strategy::RemotePut,
      Strategy::PipedLocalCompress,
      Strategy::PipedLocalGet,
      Strategy::PipedLocalPut,
      Strategy::Cleaner,
      Strategy::RbdSweeper,
      Strategy::Sweeper
    ].freeze
  end
end
